%%%%%%%% collate.m
% 17-24 December 2003

% get to the directory containing RawSpectra for the 24 low mass scans of
% the QC spectra on WCX2 chips from Henry Kuerer's study.
cd m:
addpath /proteomics/bin/Cromwell
addpath /Proteomics/bin/RWT-windows
addpath /Proteomics/Kuerer/QualityControl/Data-WCX2-LMW/AnalysisScripts
cd /Proteomics/Kuerer/QualityControl/Data-WCX2-LMW
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% RUN ONCE
% This part of the file is only used once, and it is described in
% the readme.txt file in the main directory. We build the MATLAB
% matrices so we can throw away the raw text versions of the data. 
[rawSpec, rawMass] = collatespectra('.', 'raw', 'one', 1);
rawMean = mean(rawSpec);
save rawMean rawMean
save rawSpec rawSpec
save rawMass rawMass

rawSpec4Splus = rawSpec';
save rawspec4splus rawSpec4Splus -V4
rawMass4Splus = rawMass';
save rawmass4splus rawMass4Splus -V4

% Note that we use S-Plus to implement the peak-finding method of
% Yasui et al. We made this decision because they explicitly use
% a super-smooth, which is already implemented in S-Plus. The script
% that performs this analysis is called "kuerer-yasui.ssc". The result
% is used as part of Figure 5 of the paper.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load the data. (Use this when we return to start again.)
% These are MAT files of the raw spectra from 24 samples
% original data is in RawXML (straight from Ciphergen software version 3.2)
% and RawSpectra (converted using ciphergenXMLreader.pl in Oct 2003)
% The names of the spectra, which include some of the critical covariates,
% are listed in the same order as the matrix rows in RawFiles.txt
cd /Proteomics/Kuerer/QualityControl/Data-WCX2-LMW
load rawSpec
load rawMass
load rawMean

% These figures are not used in the paper. They are, however, mentioned
% as 'data not shown'.
imagesc(log(1+rawSpec))
print -dpng ./Figures/rawLogHeatMap.png
imagesc(rawSpec)
print -dpng ./Figures/rawHeatMap.png

plot(rawSpec')
axis([0 20000 0 17000])
xlabel 'Clock Tick'
ylabel Intensity
print -dpng ./Figures/rawSpectraClockTick.png

axis([0 5000 0 17000])
print -dpng ./Figures/rawSpectraSaturation.png

axis([5500 12000 500 12500])
print -dpng ./Figures/rawSpectraBaselines.png


plot(rawMass, rawSpec')
axis([-300 30000 0 17000])
xlabel 'Mass/Charge'
ylabel 'Intensity'
print -dpng ./Figures/rawSpectra.png


plot(rawMass, rawMean)
axis([0 2000 5000 17000])
axis([1000 2000 10000 16000])
% Saturation is pretty obvious out to 800 or so daltons/charge.
% Things look okay after about 1000 daltons.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% get some understanding of the algorithm and its options
% pick a sample spectrum to work with (no special reason for this choice)
specid = 11;
spectrum = rawSpec(specid,:);

L = 10;				% power of 2 used for the wavelet transform
% spectrum length must be a multiple of this power of 2
needs = ceil(length(spectrum)/2.^L)*2.^L - length(spectrum)
% we pad by reflecting the end of the spectrum
% don't forget to mention this in the paper!
spectrum = [spectrum spectrum(end:-1:end-needs+1)];
length(spectrum)

h = daubcqf(8);	% base wavelet; decided on degree 8 previously
[yl yh] = mrdwt(spectrum, h, L);
median(abs(yh))/0.67		% 59.5	Should use the MAD (option(3) == 0)
% don't forget to metion the magic factor of two-thirds in the paper!
std(yh)						% 9493	Don't use the SD

thld = 6*median(abs(yh))/0.67;	% 357

xhsoft = SoftTh(yh, thld);
xlsoft = SoftTh(yl, thld);
xhhard = HardTh(yh, thld);
xlhard = HardTh(yl, thld);
% can't tell yet whether hard or soft thresholding matters

xdhard = mirdwt(xlhard, xhhard, h, L);
xdsoft = mirdwt(xlsoft, xhsoft, h, L);

% now it is clear that hard thresholding is better than soft thresholding
% this figure is not included in the paper; it is referred to as 'data not
% shown'
hold off
plot(spectrum, 'g')
hold on
plot(xdhard)
plot(xdsoft, 'r')
title 'Hard (blue) or Soft (red) thresholding'
axis([6901 7400 4001 10000])
print -dpng ./Figures/hardOrSoft.png

% The rest of this sectionj is exploratory, and does not make it into
% paper in any form.

% also want to know whether or not to threshold the low pass part
% has absolutely no effect on this sample, so we won't worry about it
xd0hard = mirdwt(yl, xhhard, h, L);
xd0soft = mirdwt(yl, xhsoft, h, L);
hold off
plot(xdhard-xd0hard)

plot(xdsoft-xd0soft)

xdlowpass = mirdwt(yl, 0*xhhard, h, L);
hold off
plot(spectrum, 'g')
hold on
plot(xdhard)
plot(xdlowpass, 'r')
% the low pass component may give us some clue about baseline, but most of the
% mess is down in the matrix region anyway. Let's not worry about it

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Already decided in previous experiments that a Daubechies degree 8 wavelet
% works fine on these data. Now see how the threshold affects things

L = 10;
specid = 1 + floor(24*rand(1));
specid=6;
spectrum = rawSpec(specid,:);
needs = ceil(length(spectrum)/2.^L)*2.^L - length(spectrum)
spectrum = [spectrum spectrum(end:-1:end-needs+1)];

[baseline40 smooth40] = waveletSmoothAndBaselineCorrect(spectrum, 40);
[baseline20 smooth20] = waveletSmoothAndBaselineCorrect(spectrum, 20);
[baseline10 smooth10] = waveletSmoothAndBaselineCorrect(spectrum, 10);
[baseline06 smooth06] = waveletSmoothAndBaselineCorrect(spectrum, 6);
[baseline04 smooth04] = waveletSmoothAndBaselineCorrect(spectrum, 4);

cut = length(rawMass);
spectrum = spectrum(1:cut);
smooth40 = smooth40(1:cut);
smooth20 = smooth20(1:cut);
smooth10 = smooth10(1:cut);
smooth06 = smooth06(1:cut);
smooth04 = smooth04(1:cut);

noise40 = spectrum - smooth40;
noise20 = spectrum - smooth20;
noise10 = spectrum - smooth10;
noise06 = spectrum - smooth06;
noise04 = spectrum - smooth04;

% We use part of the next set of figures as Figure 2 of the paper:
% a medium view and a closeup view oft e spectrum and its smoothed relatives.
figure(1)
set(gcf, 'PaperType', 'arch-A')	% 9 x12
% we'll treat it as 3x4 for resolution purposes

subplot(111)
hold off
i0 = plot(rawMass, spectrum, 'c');
set(gca, 'FontSize', 20)
xlabel 'Mass/Charge'
hold on
i04 = plot(rawMass, smooth04, 'm');
i10 = plot(rawMass, smooth10, 'r');
i06 = plot(rawMass, smooth06, 'g');
set(i10, 'LineWidth', 2, 'Color', [1 0 0])
set(i06, 'LineWidth', 2, 'Color', [0 0 1])
set(i04, 'LineWidth', 2, 'Color', 0.6*[0 1 0])
set(i0, 'Color', 0.3*[1 1 1])

% Figure 2
axis([2801 3300 2851 3800])
print -dtiff -r300 ./Figures/fig2-smoothClose.tif
print -dpng ./Figures/smoothClose.png

i20 = plot(rawMass, smooth20, 'g');
set(i20, 'LineWidth', 2, 'Color', 0.75*[0 1 0])
i40 = plot(rawMass, smooth40, 'g');
set(i40, 'LineWidth', 2, 'Color', 0.75*[0 1 1])

axis([8501 18000 601 1500])
print -dpng ./Figures/smoothBroad.png
axis([8501 10000 1001 1500])
print -dpng ./Figures/smoothMedium.png
axis([9701 10100 1601 1900])
axis([7501 8100 1601 2200])
axis([0 20000 0 17000])

% this is the code to create figure 1 of the paper: a plot of the
% residuals (noise) after the wavelet smooth.
subplot(511)
plot(noise40, 'k')
title 'Threshold 40'
subplot(512)
plot(noise20, 'k')
title 'Threshold 20'
subplot(513)
plot(noise10, 'k')
title 'Threshold 10'
subplot(514)
plot(noise06, 'k')
title 'Threshold 6'
subplot(515)
plot(noise04, 'k')
title 'Threshold 4'
xlabel 'Clock Tick'
print -dtiff -r300 ./Figures/fig1-NoiseByThreshold.tif
print -dpng ./Figures/lowMassScanNoiseByThreshold.png


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% RUN ONCE
% Now let's do the basic processing. Here we rely on another script that will
% do the computations and save the results; it's called basecorr2.m and it lives
% in the current directory.

L = 10;		% should be the same value as above; we'll eventually make this constant
M = 3500;	% number of clock ticks to ignore. In this data set, about 200 daltons
thld = 10;	% multiple of MAD at which to do wavelet thresholding; slightly oversmooths

% Note the padding by reflecting the end of the spectrum
% We chop back to the original length before saving the data
raw = [rawSpec rawSpec(:, end:-1:end-needs+1)];

cd /Proteomics/Kuerer/QualityControl/Data-WCX2-LMW
basecorr2(raw, rawMass, M, L, thld);

% At this point, we have created and saved the following *.mat files
%	noises.mat					% matrix of noise = raw - waveletSmoothed
%	baselineCorrected.mat	% matrix of baseline corrected (bc) spectra
%	meanBase.mat				% vector mean (bc) spectra
%	normalized.mat				% matrix of total-ion-current normalized bc spectra
%	meanNorm.mat				% vector mean of normalized spectra
%	peakLocations.mat			% matrix of peak locations in clock ticks
%	numpeaks.mat				% vector of number of peaks per spectrum

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reload the data to start again

load noises
load baselineCorrected
load meanBase
load normalized
load meanNorm
load peakLocations
load numpeaks

% This is Figure 3 of the paper: it shows that the noise is about the
% same across spectra and mostly uncorrelated.
subplot(111)
set(gca, 'FontSize', 14)
hold off
imagesc(noises)
set(gca, 'FontSize', 16)
colorbar horiz
colormap gray
xlabel 'Time'
ylabel 'Spectrum Index'
print -dtiff -r300 ./Figures/fig3-noiseBySample-arrow.tif
print -dpng ./Figures/noiseBySample.png

% we look at a sample spectrum. ignore the fact that 13 and 11 aren't equal.
ii = 11
raw13 = rawSpec(ii,:);
bc13 = bc(ii,:);
nc13 = nc(ii,:);
pk13 = peaks(ii, find(peaks(ii,:)>0));
noise13 = noises(ii,:);

% we did not use this figure in the paper
hold off
plot(raw13)
hold on
temp = plot(bc13, 'r');
set(temp, 'LineWidth', 2, 'Color', 0.75*[0 1 1])
axis([3501 18000 0 17000])
xlabel 'Clock Tick'
ylabel 'Intensity'
print -dpng ./Figures/baselineBroad.png

% we did use the close-up view as Figure 4 of the paper. we'll remain silent
% about the flat intensity shift; it only serves to bring the two graphs
% close enough together to make the details visible.
hold off
plot(rawMass, raw13-1500, 'k')
hold on
temp=plot(rawMass, bc13, 'k');
set(temp, 'LineWidth', 2)
%set(temp, 'Color', 0.75*[0 1 1])
axis([4931 8180 0 2800])
xlabel 'Mass'
ylabel 'Intensity'
print -dtiff -r300 ./Figures/fig4-baselineClose.tif
print -dpng ./Figures/baselineClose.png

% This figure is not used in the paper. It illustrates how we smoothed the
% local noise estimaets using a fixed-width window.
x=abs(noise13);
y = 0;
for i = 1:length(x)-500,
   y(i) = mean(x(i:i+500));
end
y=[repmat(y(1), [1 250]) y repmat(y(end), [1 250])];
hold off
plot(x, 'y.')
hold on
plot(y, 'k')
print -dpng ./Figures/localNoiseEstimates.png

% Now we compute the signal-to-noise ratios at each clock tick in all
% 24 spectra.
xx=abs(noises);
[rx cx] = size(xx)
yy = repmat(0, [rx cx-500]);
for i = 1:cx-500,
   yy(:, i) = mean(xx(:, i:i+500)')';
end
yy=[repmat(yy(:, 1), [1 250]) yy repmat(yy(:, end), [1 250])];
sig2noise = bc ./ yy;
save sig2noise sig2noise -V4

sn13 = sig2noise(ii,:);


% Now we prepare a data structure that lays out all the peak information
% in a coherent fashion. The peak matrix at present contains one row for
% each spectrum, and enough columns to hold teh maximum number of peaks
% found in any single spectrum. Each entry in this matrix is the clock
% tick location of a peak in that spectrum. We march through the peak
% matrix, creating a new matrix with one row for each individual peak
% in each spectrum, with the five columns:
%	spectrum index, peak index, peak mass, and signal-to-noise ratio, norm intensity
[rp cp] = size(peaks);	% 24, 237
nump=0;
peakRow = [0 0 0 0 0];
for i=1:rp,
   for j=1:cp,
      if peaks(i,j) > 0,
         nump = nump + 1;
         p = peaks(i,j);
         peakRow(nump,:) = [i p rawMass(p) sig2noise(i, p) nc(i, p)]; 
      end
   end
end
% For some purposes, we prefer to have the peaks sorted in order
% of decreasing S/N.
[y i] = sort(peakRow(:,4));
peakRow = peakRow(i(end:-1:1),:);
% Most of the time, however, we prefer to have the peaks sorted in
% order of increasing mass.
[y i] = sort(peakRow(:, 2));
pbm = peakRow(i,:);


% Now we write out the data in text format, with the columns separated
% by tabs. This lets us read the data into perl, for example, where we
% can use hashes more conveniently for the next step.
[fid, message] = fopen('./peakRow.txt', 'w')
fprintf(fid, '%d\t%d\t%9.2f\t%9.2f\r\n', pbm(:,1:4)')
fclose(fid)

% The perl script that reads this data and processes the result is called
% 'collectPeaks.pl'. The point is to coalesce the peaksacross spectra, so
% we can determine which peaks represent the same substance. We eventually
% decided on a two-pass procedure. We start with peaks with S/N > 10. We
% group them if they differ in location by at most 7 ticks or in mass by
% at most 0.3%. We then add peaks to these defined clusters as long as
% S/N > 2 and they lie within 7 ticks or 0.3% of the mass of a known peak.
%
% THIS SHOULD BE REIMPLEMENTED IN MATLAB TO MAKE THIS SCRIPT SELF-CONTAINED.

% This figure is used for Figure 8 of the paper.
hold off
temp = plot(rawMass, rawSpec', 'k');
set(temp, 'LineWidth', 2)
%plot(rawMass, rawSpec')
hold on
plot([2685 2685], [3001 14500], 'k')
axis([2621 2760 3001 14500])
xlabel Mass
ylabel Intensity
print -dtiff -r300 ./Figures/fig8-peak10.tif
print -dpng ./Figures/peak10.png

% We're still deciding whether to use the next figure in the paper.
% first step is to interpolate from a constant time step to a constant
% mass step so we can display artificial gel images along the mass axis.
mx = massTrans(log(0.1+nc), rawMass, 1);
% we also want to indicate peak locations.
bigpk13 = pk13(sn13(pk13) > 10);
smpk13 = pk13(sn13(pk13) > 2 & sn13(pk13) < 10);

% this version marks the peaks from one spectrum. It would be better
% to mark the merged peaks across spectra. To do that, we either need
% to read in the merge information from the perl processing or we need
% to reimplement the processing in MATLAB.
hold off
imagesc(5-mx)
colormap gray
colorbar horiz
hold on
for i = 1:length(bigpk13),
   m = rawMass(bigpk13(i));
   plot([m m], [1 24], 'r')
end
for i = 1:length(smpk13),
   m = rawMass(smpk13(i));
   plot([m m], [1 24], 'g')
end
plot(rawMass, 24-rawMean/750, 'b')

% so okay, we bit the bullet and reimplemented the perl code in MATLAB.
% once the algorithm had settled down, it turned out we really didn't
% need the hashes after all.
[a, b] = peakSet(pbm, 10, 2, 7, 0.003);
save all_174_Peaks a -V4

n0 = a(:,6);
mean0 = n0 .* a(:,12) / 24;
var0 = (n0-1)/23 .* a(:,13).^2 + (n0).*(24-n0)/(24*23) .*  a(:,12).^2;
cv0 = sqrt(var0) ./ mean0;

unicounter = a(:,6);

meanlog = a(:,12);
stdlog = a(:,13);
cv = stdlog./meanlog;
min3 = unicounter > 2;
massloc = (a(:,4)+a(:,5))/2;

hold off
plot(meanlog(min3), stdlog(min3), 'k.')
xlabel 'Mean Log Intensity'
ylabel 'Standard Deviation'

plot(massloc(min3), cv(min3), '.')
xlabel Mass
ylabel 'Coefficient of Variation'

mean(cv(min3))
min(cv(min3))
max(cv(min3))

found24 = rawMass(a(unicounter == 24, 11));
found20 = rawMass(a(unicounter < 24 & unicounter > 19, 11));
found10 = rawMass(a(unicounter < 20 & unicounter > 9, 11));
found01 = rawMass(a(unicounter < 10, 11));
% 
yarg = gray(256);
yarg = yarg(end:-1:1,:);
size(yarg)

%% Figure 9 of the paper
hold off
imagesc(mx)
colormap(yarg)
colormap jet
colorbar horiz
hold on
for i = 1:length(found24),
   m = found24(i);
   temp = plot([m m], [1 24], 'w');
%   set(temp, 'LineWidth', 2, 'Color', [1 0 0])
%   set(temp, 'LineWidth', 2, 'Color', [0 0 0])
   set(temp, 'LineWidth', 2)
end
for i = 1:length(found20),
   m = found20(i);
   temp = plot([m m], [1 24], 'w');
%   set(temp, 'LineWidth', 2, 'Color', [1 0 0.5])
%   set(temp, 'LineWidth', 2, 'Color', [0 0 0])
   set(temp, 'LineWidth', 2)
end
for i = 1:length(found10),
   m = found10(i);
   temp = plot([m m], [1 24], 'w');
%   set(temp, 'LineWidth', 2, 'Color', [1 0 1])
%   set(temp, 'LineWidth', 2, 'Color', [0 0 0])
   set(temp, 'LineWidth', 2)
end
%plot(rawMass, 24-rawMean/750, 'w')
temp = plot(rawMass, 2*(24-rawMean/750)-3.2, 'w');
set(temp, 'LineWidth', 3, 'Color', [1 1 0])
set(temp, 'LineWidth', 3, 'Color', [1 1 1])
xlabel 'Mass'
ylabel 'Spectrum Index'

axis([1350 1800 1 24])
print -dtiff -r300 ./Figures/fig9-overlaidGel.tif
print -dpng ./Figures/overlaidGel.png


[a, b, c, d, e, f, g, h, i]= textread('kuererMaxSens.tsv', '%d%d%f%f%f%f%f%f%f', 'delimiter', ',', 'headerlines', 1);
ticker = 0*d;
for k = 1:length(d),
   ticker(k) = sum(rawMass < d(k));
end
cipher = [a ticker d e c];

[y z] = sort(d);
cipher = cipher(z,:);

[a, b] = peakSet(pbm, 10, 2, 7, 0.003);
unicounter = a(:,6);

[c, d] = peakSet(cipher, 0, 0, 7, 0.003);
cunicounter = c(:,6);



subplot(211)
colormap gray
hist(unicounter, 1:24)
xlabel 'Number of Spectra'
ylabel 'Number of Peaks'
title 'Peak Distribution Using Our Algorithm'
subplot(212)
hist(cunicounter, 1:24)
xlabel 'Number of Spectra'
ylabel 'Number of Peaks'
title 'Peak Distribution Using Ciphergen Algorithm'
print -dtiff -r300 ./Figures/fig7-peakHistograms.tif
print -dpng ./Figures/peakHistograms.png

acent = floor((a(:,2)+a(:,3))/2);
ccent = floor((c(:,2)+c(:,3))/2);
aa = acent(a(:,6)>10);
ab = acent(a(:,6)<=10);
ca = ccent(c(:,6)>10);
cb = ccent(c(:,6)<=10);
hold off
plot(rawMass, rawMean)
hold on
plot(rawMass(aa), rawMean(aa), 'r.')
plot(rawMass(ca), rawMean(ca), 'mo')
plot(rawMass(ab), rawMean(ab), 'k.')
plot(rawMass(cb), rawMean(cb), 'go')

load yasuiPeaks
yasui = data;
clear data

[s1 s2] = size(yasui);
sn0 = zeros(s1, 1);
ht0 = sn0;
for k = 1:s1,
   sn0(k) = sig2noise(yasui(k,1), yasui(k,2));
   ht0(k) = nc(yasui(k,1), yasui(k,2));
end

yasui = [yasui sn0 ht0];
[y z] = sort(yasui(:,3));
yasui = yasui(z,:);

[y, z] = peakSet(yasui, 0, 0, 7, 0.003);
yunicounter = y(:,6);
[y, z] = peakSet(yasui, 1, 1, 7, 0.003);
yunicounter2 = y(:,6);

subplot(211)
hist(yunicounter, 1:24)
title 'All 261 Peaks'
subplot(212)
hist(yunicounter2, 1:24)
title '188 Peaks with S/N > 1'
print -dpng ./Figures/yasuiPeakHistogram.png

[rp cp] = size(peaks);	% 24, 237
nump=0;
peakRow = [0 0 0 0 0];
for i=1:rp,
   for j=1:cp,
      if peaks(i,j) > 0,
         nump = nump + 1;
         p = peaks(i,j);
         peakRow(nump,:) = [i p rawMass(p) sig2noise(i, p) nc(i, p)]; 
      end
   end
end
% For some purposes, we prefer to have the peaks sorted in order
% of decreasing S/N.
[y i] = sort(peakRow(:,4));
peakRow = peakRow(i(end:-1:1),:);
% Most of the time, however, we prefer to have the peaks sorted in
% order of increasing mass.
[y i] = sort(peakRow(:, 2));
pbm = peakRow(i,:);
